**==sterab.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      SUBROUTINE STERAB(N,JMAX,DELE)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c DenSum: a code for calculating sums and densities of states.
c Copyright (C) 2002 John R. Barker 
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the "ReadMe" file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
C
C
C    CALCULATES COMPLEXIONS, DENSITIES, AND SUMS OF STATES ACCORDING TO
C     S.E. STEIN AND B.S. RABINOVITCH, J.CHEM.PHYS. 58, 2438 (1973)
C
C     RETURNS SUMS AND DENSITIES OF RO/VIB STATES IN
C           ARRAYS SUM AND DENS
C
C     DELE = GRAIN SIZE (CM-1)
C
C     ALL ENERGIES AND FREQUENCIES IN WAVENUMBERS
C
C	N	= degrees of freedom
C	JMAX	= index of energy increment corresponding to Emax
C	DELE	= energy grain (cm-1)
C
C
C	OTHER VARIABLES:
C	IDOF	= type of degree of freedom
C		= VIB (Morse or harmonic vibration)
C		= ROT (classical free rotation)
C		= QRO (quantized free rotation)
C               = BOX (quantized particle-in-a-box)
C               = HRx (hindered rotation; x=a,b,c; see below)
C               = TRN (relative translation)
C	WE	= vibrational frequency (cm-1), or moment of inertia (amu Ang^2)
C	ANH	= vector of anharmonicities, or rotor symmetry number
C	NG	= vib. degeneracy, or rotor dimension
C	SUM	= sums of states
C	DENS	= densities of states
C       zzpe    = zero point energy (vibrations and hindered rotors)
C
C     Free Rotations:
C         WE(I) is moment of inertia (amu Ang^2)
C         ANH(I) is rotor foldedness (symmetry number)
C         NG(I) is rotor dimension (renamed NDIM)
C
C     1-D Hindered Rotations:
C         B(I) is rotational constant (cm-1)
C         W(I) small vibration frequency (cm-1)
C         NG(I) is internal rotor potential energy periodicity (foldedness)
C         NSIG(I) is internal rotor symmetry number
C         VV(I) is hindrance barrier height (cm-1)
C
C     Vibrations:
C         IF ANH=0.0,  HARMONIC OSCILLATORS ASSUMED
C            NG(I) IS OSCILLATOR DEGENERACY (OR ROTOR DIMENSION);
C         BOX:  NG(I) is oscillator degeneracy
C
C      JMAX IS NUMBER OF GRAINS AND CANNOT EXCEED 20001
C
C
C
C     THIS ROUTINE USES SUBROUTINES MORLEV(....)
C                                   BOXLEV(...)
C                                   GAM(X)
C                                   ROTLEV
C                                   HRLEV
C
c *************************************************************************

      INCLUDE 'declare.inc'

      INTEGER NTRN , NTRNFLAG , JK
      DOUBLE PRECISION FACTR , atr
      EXTERNAL GAM
!      DIMENSION AT(NMAX) , T(NMAX) , IR(NMAX)

	Double precision AT(NMAX) , T(NMAX)  
	Integer IR(NMAX)

      SAVE 
C
C          INITIALIZE T AND AT
C
      T(1) = 1.0D+00
      AT(1) = 0.0D+00
      DO 100 I = 2 , JMAX
         AT(I) = 0.0D+00
         T(I) = 0.0D+00
 100  CONTINUE
C
C          TROE METHOD FOR CLASSICAL ROTATIONS, EXTENDED TO TRANSLATIONS
C          [Astholz, Troe, and Wieters, J.Chem.Phys., 70, 5107, 1979]
C          [Equation numbers from Robinson & Holbrook, Unimolecular Reactions (Wiley, 1973)]
c
      NR = 0
      NTRN = 0
      NTRNFLAG = 0
      FAC = 1.D+00
      NSYMM = 1
      DO 200 I = 1 , N                                        ! TEST FOR CLASS. ROTORS AND TRANSLATION
         IF ( IDOF(I).EQ.ROT ) THEN
            IF ( NG(I).LE.0 ) NG(I) = 1   ! free rotor dimension
            NDIM = NG(I)                  ! free rotor dimension
            BB = 16.85763D+00/WE(I)       ! rotational constant (cm-1)
            NR = NR + NDIM                ! number of classical free rotor degrees of freedom
            RG = DBLE(NDIM)/2.D+00        ! in Eq. 5.16
            BG = SQRT( BB**NDIM )         ! in Eq. 5.16
            FAC = FAC*GAM(RG)/(BG*ANH(I)) ! in Eq. 5.16; ANH(I) is free rotor foldedness (symmetry number)
            NSYMM = NSYMM*ANH(I)          ! symmetry number
         ENDIF
         IF (IDOF(I) .EQ. TRN) THEN
            NTRN = I
            NTRNFLAG = NTRNFLAG + 1       ! Flag for presence of relative translation
            IF (NTRNFLAG .GT. 1) THEN
              STOP '**** ONLY ONE 3-D TRANSLATION ALLOWED ***'
            ENDIF
         ENDIF
 200  CONTINUE
 
       IF ( (NR.GT.0) .AND. (NTRN .EQ. 0) ) THEN              ! CLASS. ROTORS BUT NO TRANSLATION
         R2 = DBLE(NR)/2.D+00             ! in Eq. 5.15
         FAC = FAC/GAM( R2+1.D+00 )       ! in Eq. 5.15
         DO 250 K = 2 , JMAX              ! Initialize T(K)
            E = (K-1)*DELE
            T(K) = FAC*( SQRT(E**NR) - SQRT( (E-DELE)**NR ) )           !  number of classical free rotor states in grain
 250     CONTINUE
         T(1) = 0.0d+00
      ELSEIF ( (NTRN .GT. 0) .AND. (NR .EQ. 0) ) THEN         ! TRANSLATION BUT NO CLASS. ROTORS
         FACTR = 2.43980d+20*( AMOM(NTRN) )**1.5d+00                      ! states per cc
         DO 251 K = 2 , JMAX               
            E = (K-1)*DELE
            T(K) = FACTR*( E**1.5d+00 - (E-DELE)**1.5d+00 )               ! number of classical translational states (per cc) in grain
 251     CONTINUE
      ELSEIF ( (NTRN .GT. 0) .AND. (NR .GT. 0) ) THEN         ! TRANSLATION WITH CLASS. ROTORS
         atr = (NR + 3.0D+00) / 2.D+00
         FACTR = 3.24332d+20*( AMOM(NTRN) )**1.5d+00                      ! states per cc
         FACTR =  FACTR*FAC / ( atr * GAM(atr) )
         DO 252 K = 2 , JMAX               
            E = (K-1)*DELE
            T(K) = FACTR*( E**atr - (E-DELE)**atr )                       ! number of classical rot-translational states (per cc) in grain
 252     CONTINUE
         T(1) = 0.0d+00
      ENDIF
C
C
C
      DO 400 I = 1 , N
         WRITE (*,99001) I , WE(I)
 
         IF ( IDOF(I).EQ.KRO ) THEN                             ! ********  1-Dim. free K-rotor *********
 
            BB = 16.85763D+00/WE(I)     ! rotational constant (cm-1)
            NDIM  = 1                   ! Rotor dimension
            NSYMM = ANH(I)              ! Symmetry number
            JK    = NG(I)               ! quantum number for total angular momentum
 
            CALL KROTLEV(DELE,JMAX,BB,JK,IR,IMAX)

            IF ( IMAX .GT. 0 ) THEN
            DO 281 J = 1 , IMAX ! J = quantum number for K-rotor;  IMAX is the number of energy levels
               IF ( J.EQ.0 ) THEN
                  F = 1         ! F is the multiplicity of the rotational level (see J.L. McHale, Molecular Spectroscopy (Prentice-Hall, 1999), 216f.
               ELSE
                  F = 2.0       ! 1-D rotor (e.g. single-axis internal rotor)
               ENDIF
               DO 261 K = 1 , JMAX
                  KARG = K - IR(J)
                  IF ( KARG.GT.0 .AND. KARG.LE.NMAX)
     &                   AT(K) = AT(K) + F*T(KARG)
 261           CONTINUE
 281        CONTINUE
            ENDIF
 
            DO 301 J = 1 , JMAX
               T(J) = AT(J) + T(J)
               T(J) = T(J) / NSYMM
               AT(J) = 0.0D+00
 301        CONTINUE
c            T(1) = 1.0d+00/NSYMM                                ! Symmetry number correction
 
 
         ELSEIF ( IDOF(I).EQ.QRO ) THEN                         ! ********  Quantized free rotors *********
 
            BB = 16.85763D+00/WE(I)     ! rotational constant (cm-1)
            NDIM = NG(I)                ! Rotor dimension
            NSYMM = ANH(I)              ! Symmetry number
 
            CALL ROTLEV(DELE,JMAX,BB,NDIM,IR,IMAX)

            DO 280 J = 1 , IMAX ! J = rot quantum number;  IMAX is the number of energy levels
               IF ( J.EQ.0 ) THEN
                  F = 1         ! F is the multiplicity of the rotational level (see J.L. McHale, Molecular Spectroscopy (Prentice-Hall, 1999), 216f.
               ELSEIF ( NDIM.EQ.1 ) THEN   ! 1-D rotor (e.g. single-axis internal rotor)
                  F = 2.0
               ELSEIF ( NDIM.EQ.2 ) THEN   ! 2-D rotor
                  F = 2.0*J + 1.0
               ELSEIF ( NDIM.EQ.3 ) THEN   ! 3-D rotor (shperical top)
                  F = (2.0*J+1.0)*(2.0*J+1.0)
               ENDIF
               DO 260 K = 1 , JMAX
                  KARG = K - IR(J)
                  IF ( KARG.GT.0 .AND. KARG.LE.NMAX)
     &                   AT(K) = AT(K) + F*T(KARG)
 260           CONTINUE
 280        CONTINUE
 
           DO 300 J = 1 , JMAX
               T(J) = AT(J) + T(J)
               T(J) = T(J) / NSYMM
               AT(J) = 0.0D+00
 300        CONTINUE
c            T(1) = 1.0d+00/NSYMM                                ! Symmetry number correction
 
         ELSEIF ( (IDOF(I).EQ.HRA) .OR. (IDOF(I).EQ.HRB) .OR. 
     &            (IDOF(I).EQ.HRC) ) THEN                       ! ******  Quantized hindered rotors ********
 
            BB = B(I)           ! rotational constant (cm-1)
            VVV = VV(I)         ! hindrance barrier (cm-1)
            NGG = NG(I)         ! Potential energy symmetry (foldedness)

		CALL SHRLEV(T,AT,DELE,JMAX,BB,VVV,NGG,1,IMAX,zap,
     &          'Vhrd1','Bhrd1',0.0d0,0.0d0)		! rigid, symmetrical hindered rotor

            zzpe(I) = zap                                        ! Zero Point Energy

	ELSEIF ( IDOF(I).EQ.HRD ) THEN

		DO L=1, NCV(I)
			CVt(L)=CV(I,L)
		ENDDO
                DO L=1, NCB(I)
                        CBt(L)=CB(I,L)
                ENDDO

		CALL UHRLEV(T,AT,DELE,JMAX,NCB(I),NCV(I),CBt,CVt,
     &          NSV(I),NSB(I),IMAX,zap,Vhr(I),Bhr(I),Phav(I),Phab(I))	! non-rigid, unsymmetrical hindered rotor


	     zzpe(I) = zap                                        ! Zero Point Energy

         ELSEIF ( IDOF(I).EQ.VIB ) THEN                         ! ********  Vibrations **********************
             CALL MORLEV(DELE,JMAX,WE(I),ANH(I),IR,IMAX,zap)
             zzpe(I) = zap                                       ! Zero Point Energy

            DO 380 L = 1 , NG(I)                                ! Loop for degenerate vibrations
               DO 350 J = 1 , IMAX                              ! IMAX is the number of energy levels
                  DO 345 K = 1 , JMAX
                     KARG = K - IR(J)
                     IF ( KARG.GT.0 .AND. KARG.LE.NMAX )
     &                      AT(K) = AT(K) + T(KARG)
 345              CONTINUE
 350           CONTINUE
 
               DO 360 J = 1 , JMAX
                  T(J) = AT(J) + T(J)
                  AT(J) = 0.0D+00
 360           CONTINUE
 380        CONTINUE
  
         ELSEIF ( IDOF(I).EQ.BOX ) THEN                         ! ********  Particle-in-a-Box ***************
            CALL BOXLEV(DELE,JMAX,WE(I),IR,IMAX,zap)
            zzpe(I) = zap                                       ! Zero Point Energy
            DO 390 L = 1 , NG(I)                                ! Loop for degenerate vibrations
               DO 355 J = 1 , IMAX                              ! IMAX is the number of energy levels
                  DO 365 K = 1 , JMAX
                     KARG = K - IR(J)
                     IF ( KARG.GT.0 .AND. KARG.LE.NMAX )
     &                   AT(K) = AT(K) + T(KARG)
 365              CONTINUE
 355           CONTINUE
 
               DO 370 J = 1 , JMAX
                  T(J) = AT(J) + T(J)
                  AT(J) = 0.0D+00
 370           CONTINUE
 390        CONTINUE

         ENDIF
 
 400  CONTINUE                ! loop quantized degrees of freedom

c
c    Reminder: T(I) is the number of states in the Ith grain.
c
      SUM(1) = T(1)                                    ! Energy at E = 0 (at TOP of first grain)
      DENS(1) = T(1) / DELE
      DO 500 I = 2 , JMAX
         SUM(I)  = T(I) + SUM(I-1)                     ! Sum of states at TOP of energy grain
         DENS(I) = T(I)/DELE
 500  CONTINUE

      RETURN

99001 FORMAT (10X,' Mode no.',I5,F10.2)

      END
