        SUBROUTINE ECKART(Mol,i,EE,psum,j)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c This subroutine calculates the sum of states of the transition state
c modified to take into account the tunneling transmission coefficient.
c This modified sum of states is defined as psum below. psum is 
c calculated as the convolution of the density of states of the transition 
c state and the tunneling transmission coefficient. The program returns 
c psum to ratearray for calculation of k(E) corrected for tunneling
c
c-----------------------------------------------------------------------------

        IMPLICIT NONE
        INCLUDE 'declare1.inc'
        INTEGER Mol , i,  j , istep , ij , ijmax
        DOUBLE PRECISION pi , e2 , e3 , econstant , vh , pre
        DOUBLE PRECISION E1 , EE , E , tstep, rij, Egrain
        DOUBLE PRECISION a1 , b1 , c1 , c2 , c3 , c4 
        DOUBLE PRECISION ptop , pbot , tprob , psum , DENTS 
        DOUBLE PRECISION a5,b5,c5,d5
        DOUBLE PRECISION term1,term2,term3,term4,Echeck
        SAVE 
     


!	constants used in applying the tunneling correction

        pi=4.0D+00*DATAN(1.0D+00)
        e2=Eor(Mol,i)**-0.5d0 
        e3=V1(Mol,i)**-0.5d0
        econstant=(e2+e3)**-1.0d0
        vh=plank*avogadro*clight*jtocm/1000.0d0
        pre=(vimag(Mol,i)*vh)**-1.0d0

        c1=2.0d0*pi
        c2=(Eor(Mol,i)*V1(Mol,i))*pre**2.0d0
        c3=1.0d0/16.0d0
        c4=c1*(c2-c3)**0.5d0

        psum=0.0d0                 !zero out sum of states (ss) array

!	Egrain will be the array (lower or upper) dependent integration step size

        if(j.le.imax1*2)then       !these lines set the integration step size
          Egrain=Egrain1           !j is the index from ratearray.f which tells
          tstep=Egrain1            !us at which energy EE we are at; for j.le.imax1*2
        else                       !we use Egrain1 to perform the integration
          Egrain=Egrain2           
          tstep=Egrain2
        end if

        ijmax = 1 + EE/Egrain
        do ij = 1 , ijmax          !only calculate ss at grain energies
          rij = (ij-1)*Egrain

          E=rij                    !energy in the molecule's other degrees of freedom

          E1=EE-E-Eor(Mol,i)       !energy in the rxn coordinate

          if(EE.lt.0.0d0)goto 18   !EE is the energy from the zpe of the reactant it 
                                   !can never be negative

          if(Eor(Mol,i).ge.V1(Mol,i))then  !endothermic and barrier-less rxn condition
            Echeck=Eor(Mol,i)-V1(Mol,i)

            if((EE.ge.Echeck).and.(E1.ge.-V1(Mol,i)))then    !tunneling corrections are undefined if
                                                             ![EE < (Eor-V1)] for endothermic reactions
              continue
            else
              goto 18
            end if

          else                              !exothermic rxn condition

            if((EE.ge.0).and.(E1.ge.-Eor(Mol,i)))then
              continue         
            else
              goto 18
            end  if
                            
          end if
  

          a1=4.0d0*pi*pre*(E1+Eor(Mol,i))**0.5d0*econstant
          b1=4.0d0*pi*pre*(E1+V1(Mol,i))**0.5d0*econstant

!	approximate sinh(x) and cosh(x) by exp(x); this approximation
!	is valid since x is the value of a1, b1 and c4, and in this case
!	the values for these variables are large 

          a5=a1
          b5=b1
          c5=((a1+b1)/2.0d0)
          d5=c4

          term1=b5-d5
          term2=c5-a5
          term3=d5-a5
          term4=c5-d5

          tprob=exp(term1)/(exp(term2)*exp(term4)+exp(term3)) ! tunneling probability

          if(tprob.le.1.0d00)then  ! all tunneling probabilities should be 1 or less
            continue
          else
            tprob=1.0d00             ! this covers the case that the exponential terms
          end if                     ! in the above expression become too large; when this
                                     ! happens (and it sometimes happens for energies much
                                     ! greater then the reaction threshold and for small imaginary
                                     ! frequency) tprob=NAN when in fact it should be equal to one.
                                     ! Setting tprob=1 takes care of this problem

        if(tprob.lt.(tunthresh))then ! if tprob is less than tunthresh the contribution to the sum
          return                     ! of states of the transition state is very small and we stop
        else                         ! peforming the convolution
          continue
        end if
          psum=psum+tstep*(tprob*DENTS(Mol,i,E)) !density of states of TS is interpolated at energy E
 18    CONTINUE
       end do 
      RETURN
      END
